From e4e4981af99e13b4676cc4763ab191189cf07467 Mon Sep 17 00:00:00 2001
From: Nicolas George <nicolas.george@normalesup.org>
Date: Wed, 4 Jul 2012 12:15:01 +0200
Subject: [PATCH] oggdec: import permute matrix from oggenc.

The previous permutations were inverted while
the code assumes they are not.

Also, the matrix from oggenc goes to 8 channels.

Signed-off-by: Nicolas George <nicolas.george@normalesup.org>
---
 oggdec/oggdec.c |   14 +++++++++++---
 1 file changed, 11 insertions(+), 3 deletions(-)

diff --git a/oggdec/oggdec.c b/oggdec/oggdec.c
index 5ffb125..f7ff208 100644
--- a/oggdec/oggdec.c
+++ b/oggdec/oggdec.c
@@ -225,13 +225,21 @@ static FILE *open_output(char *outfile)
 static void
 permute_channels(char *in, char *out, int len, int channels, int bytespersample)
 {
-    int permute[6][6] = {{0}, {0,1}, {0,2,1}, {0,1,2,3}, {0,1,2,3,4}, 
-        {0,2,1,5,3,4}};
+    int permute[6][6] = {
+        {0},              /* 1.0 mono   */
+        {0,1},            /* 2.0 stereo */
+        {0,2,1},          /* 3.0 channel ('wide') stereo */
+        {0,1,2,3},        /* 4.0 discrete quadraphonic */
+        {0,2,1,3,4},      /* 5.0 surround */
+        {0,2,1,4,5,3},    /* 5.1 surround */
+        {0,2,1,5,6,4,3},  /* 6.1 surround */
+        {0,2,1,6,7,4,5,3} /* 7.1 surround (classic theater 8-track) */
+    };
     int i,j,k;
     int samples = len/channels/bytespersample;
 
     /* Can't handle, don't try */
-    if (channels > 6)
+    if (channels > 8)
         return;
 
     for (i=0; i < samples; i++) {
-- 
1.7.10

